module lexer;
import std.ascii;
import std.string;

enum Type{
  nope, identifier, number, string, _true, _false,
  
  //Operators (+,-,*,/,=)
  plus, minus, times, divide, equals,
  
  semicolon
}

struct Token{
  Type type;
  string value;
}

class Lexer{
  Token[] lexit(string code){
    Token[] tokens;
    auto cp = code.ptr;
    auto end = code.ptr + code.length;
    while(cp < end){
      if(isWhite(*cp)){
	cp++;
      }else if(isAlpha(*cp)){
	/* Harf ve sayı olduğu sürece bunları tek tek dizi içerisine atıyoruz */
	string tmp;
	do{
	  tmp ~= *cp;
	  cp++;
	} while(cp < end && isAlphaNum(*cp));
	// Eğer elde edilen kelime bir anahtar kelime ise bunu bir değişken yerine anahtar kelime olarak kayıt ediyor.
	if(tmp == "true") tokens ~= Token(Type._true, tmp);
	else if(tmp == "false") tokens ~= Token(Type._false, tmp);
	else tokens ~= Token(Type.identifier, tmp);
      }else if(isDigit(*cp)){
	//Rakamları yakalıyoruz.
	string tmp;
	do{
	  tmp ~= *cp;
	  cp++;
	} while(cp < end && isDigit(*cp));
	tokens ~= Token(Type.number, tmp);
      }else if(*cp == '\"'){
	// "metin" şeklindeki bir stringi elde ediyoruz.
	string tmp;
	cp++;
	do{
	  tmp ~= *cp;
	  cp++;
	} while(cp < end && *cp!='\"'); //Tırnak işareti gelene kadarki bütün karakterleri diziye atıyoruz.
	cp++;
	tokens ~= Token(Type.string, tmp);
      }else if(*cp == '+'){
	tokens ~= Token(Type.plus);
	cp++;
      }else if(*cp == '-'){
	tokens ~= Token(Type.minus);
	cp++;
      }else if(*cp == '*'){
	tokens ~= Token(Type.times);
	cp++;
      }else if(*cp == '/'){
	cp++;
	/* "//" şeklindeki yorum satırlarını görmezden gel. */
	if(*cp == '/'){
	  while(cp < end && *cp != '\n'){
	    cp++;
	  }
	  cp++;
	}else tokens ~= Token(Type.divide);
      }else if(*cp == '='){
	tokens ~= Token(Type.equals);
	cp++;
      }else if(*cp == ';'){
	tokens ~= Token(Type.semicolon);
	cp++;
      }else{
	throw new Exception("Beklenmeyen karakter: %s".format(*cp));
      }
    }
    return tokens;
  }
}